@@ -4,7 +4,7 @@ import numpy as np
 
 import wx
 from wx.lib import masked
-
+from floatcontrol import FloatCtrl
 from wxmplot import PlotPanel
 
 def next_data():
@@ -17,8 +17,7 @@ def next_data():
 
 class StripChartFrame(wx.Frame):
     def __init__(self, parent, ID, **kws):
-
-        kwds["style"] = wx.DEFAULT_FRAME_STYLE|wx.RESIZE_BORDER|wx.TAB_TRAVERSAL
+        kws["style"] = wx.DEFAULT_FRAME_STYLE|wx.RESIZE_BORDER|wx.TAB_TRAVERSAL
 
         wx.Frame.__init__(self, parent, ID, '',
                          wx.DefaultPosition, wx.Size(-1,-1), **kws)
@@ -60,12 +59,8 @@ class StripChartFrame(wx.Frame):
         b_off.Bind(wx.EVT_BUTTON, self.onStopTimer)
 
         tlabel = wx.StaticText(btnpanel, -1, '  Time range:')
-        self.time_range = masked.NumCtrl(btnpanel, -1,
-                                         value=abs(self.tmin),
-                                         name="target control",
-                                         size=(75, -1),
-                                         fractionWidth=1,  allowNegative=False,
-                                         allowNone=False, min=0.1, limited=True)
+        self.time_range = FloatCtrl(btnpanel,  size=(75, -1),
+                                    value=abs(self.tmin), precision=1)
 
         btnsizer.Add(b_on,   0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 0)
         btnsizer.Add(b_off,  0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 0)
@@ -97,6 +92,7 @@ class StripChartFrame(wx.Frame):
         t0,y0 = next_data()
         self.ylist = [y0]
         self.tlist = [t0]
+        self.tmin_last = -10000
         self.time0    = time.time()
         self.timer.Start(50)
 
@@ -118,12 +114,20 @@ class StripChartFrame(wx.Frame):
         if n <= 2:
             self.plotpanel.plot(tdat, ydat)
         else:
-            self.plotpanel.update_line(0, tdat, ydat)
+            self.plotpanel.update_line(0, tdat, ydat, draw=True)
             self.write_message(" %i points in %8.4f s" % (n,etime))
 
-        self.plotpanel.set_xylims(((-abs(self.tmin), 0),
-                                   (ydat[mask].min(), ydat[mask].max())),
-                                  autoscale=False)
+        lims = self.plotpanel.get_viewlimits()
+        try:
+            ymin, ymax = ydat[mask].min(), ydat[mask].max()
+        except:
+            ymin, ymax = ydat.min(), ydat.max()
+        tmin = max(int(min(tdat)) - 1.0, -self.tmin)
+        if (ymin < lims[2] or ymax >  lims[3] or
+            tmin != self.tmin_last or time.time()-self.last_update > 2):
+            self.tmin_last = tmin
+            self.last_update = time.time()
+            self.plotpanel.set_xylims((tmin, 0, ymin, ymax))
 
     def OnAbout(self, event):
         dlg = wx.MessageDialog(self, "wxmplot example: stripchart app",
@@ -134,7 +138,7 @@ class StripChartFrame(wx.Frame):
     def OnExit(self, event):
         self.Destroy()
 
-app = wx.PySimpleApp()
+app = wx.App()
 f = StripChartFrame(None,-1)
 f.Show(True)
 app.MainLoop()
