@@ -42,10 +42,10 @@ for k,v in (('solid','-'), ('dashed','--'), ('dotted',':'), ('dash-dot','-.')):
     StyleMap[k]=v
 
 
-for k,v in (('no symbol','None'), ('square','s'), ('+','+'),
-            ('o','o'), ('x','x'), ('diamond','D'), ('thin diamond','d'),
-            ('^','^'), ('v','v'), ('>','>'), ('<','<'), ('|','|'),('_','_'),
-            ('hexagon','h'), ('pentagon','p'),
+for k,v in (('no symbol','None'), ('+','+'), ('o','o'), ('x','x'),
+            ('square','s'), ('diamond','D'), ('thin diamond','d'),
+            ('^','^'), ('v','v'), ('>','>'), ('<','<'),
+            ('|','|'),('_','_'), ('hexagon','h'), ('pentagon','p'),
             ('tripod 1','1'), ('tripod 2','2')):
     MarkerMap[k] = v
 
@@ -56,9 +56,9 @@ class LineProperties:
     to  make the matplotlib calls to set the Line2D properties
     """
 
-    def __init__(self,color='black',style='solid', drawstyle='default',
-                 linewidth=2,
-                 marker='no symbol',markersize=6,markercolor=None,label=''):
+    def __init__(self, color='black', style='solid', drawstyle='default',
+                 linewidth=2, marker='no symbol',markersize=6,
+                 markercolor=None, label=''):
         self.color      = color
         self.style      = style
         self.drawstyle  = drawstyle
@@ -67,6 +67,7 @@ class LineProperties:
         self.markersize = markersize
         self.markercolor= markercolor
         self.label      = label
+        self.data_range = [None, None, None, None]
 
     def update(self, line=None):
         """ set a matplotlib Line2D to have the current properties"""
@@ -138,7 +139,7 @@ class LineProperties:
         if line:
             line[0].set_markersize(self.markersize/2.0)
 
-    def set_linewidth(self,linewidth,line=None):
+    def set_linewidth(self, linewidth, line=None):
         self.linewidth=linewidth
         if line:
             for l in line:
@@ -155,12 +156,14 @@ class PlotConfig:
         self.styles      = StyleMap.keys()
         self.drawstyles  = DrawStyleMap.keys()
         self.symbols     = MarkerMap.keys()
-        self.legend_locs = ['upper right' , 'upper left', 'upper center',
-                            'lower right',  'lower left', 'lower center',
-                            'center left',  'center right', 'right', 'center']
+
+        self.legend_locs = ['upper right' , 'lower right', 'center right',
+                            'upper left', 'lower left',  'center left',
+                            'upper center', 'lower center', 'center']
+
         self.legend_onaxis_choices =  ['on plot', 'off plot']
         self.set_defaults()
-        
+
     def set_defaults(self):
         self.zoom_x = 0
         self.zoom_y = 0
@@ -170,6 +173,16 @@ class PlotConfig:
         self.xlabel = ' '
         self.ylabel = ' '
         self.y2label = ' '
+        self.plot_type = 'lineplot'
+        self.cursor_mode = 'zoom'
+        self.scatter_size = 6
+        self.scatter_normalcolor = 'blue'
+        self.scatter_normaledge  = 'blue'
+        self.scatter_selectcolor = 'red'
+        self.scatter_selectedge  = 'red'
+        self.scatter_data = None
+        self.scatter_coll = None
+        self.scatter_mask = None
 
         self.legend_loc    =  'upper right'
         self.legend_onaxis =  'on plot'
@@ -181,9 +194,9 @@ class PlotConfig:
         f0 =  FontProperties()
         self.labelfont = f0.copy()
         self.titlefont = f0.copy()
-        self.labelfont.set_size(12)
-        self.titlefont.set_size(14)
-
+        self.labelfont.set_size(9)
+        self.titlefont.set_size(10)
+        self.textcolor = '#000000'
         self.grid_color = '#E5E5E5'
         # preload some traces
         self.ntrace = 0
@@ -228,66 +241,168 @@ class PlotConfig:
         self.traces[n] = line
 
     def __mpline(self, trace):
-        n = max(0,int(trace))
+        n = max(0, int(trace))
         while n >= len(self.traces):
             self.traces.append(LineProperties())
         try:
             return self.lines[n]
         except:
-            return None
+            return self.lines[n-1]
+
+    def __gettrace(self, trace):
+        if trace is None:
+            trace = self.ntrace
+        while trace >= len(self.traces):
+            self.traces.append(LineProperties())
+        return trace
 
     def relabel(self, xlabel=None, ylabel=None, y2label=None, title=None):
         " re draw labels (title, x,y labels)"
         n = self.labelfont.get_size()
+
         rcParams['xtick.labelsize'] =  rcParams['ytick.labelsize'] =  n
+        rcParams['xtick.color'] =  rcParams['ytick.color'] =  self.textcolor
+
         if xlabel is not None:  self.xlabel = xlabel
         if ylabel is not None:  self.ylabel = ylabel
         if y2label is not None: self.y2label = y2label
         if title is not None:   self.title = title
 
         axes = self.canvas.figure.get_axes()
-        axes[0].set_xlabel(self.xlabel, fontproperties=self.labelfont)
-        axes[0].set_ylabel(self.ylabel, fontproperties=self.labelfont)
-        axes[0].set_title(self.title,   fontproperties=self.titlefont)
-        if len(axes) > 1:
-            axes[1].set_ylabel(self.y2label, fontproperties=self.labelfont)
+        kws = dict(fontproperties=self.titlefont, color=self.textcolor)
+        axes[0].set_title(self.title, **kws)
+        kws['fontproperties'] = self.labelfont
 
+        if len(self.xlabel) > 0 and self.xlabel not in ('', None, 'None'):
+            axes[0].set_xlabel(self.xlabel, **kws)
+
+        if len(self.ylabel) > 0 and self.ylabel not in ('', None, 'None'):
+            axes[0].set_ylabel(self.ylabel, **kws)
+        if (len(axes) > 1 and len(self.y2label) > 0 and
+            self.y2label not in ('', None, 'None')):
+            axes[1].set_ylabel(self.y2label, **kws)
+
+
+        def showobj(o):
+            print '============ ', o
+            for i in dir(o):
+                if i.startswith('get'):
+                    print i, getattr(o, i)
+            print '============'
+
+        for ax in axes[0].xaxis, axes[0].yaxis:
+            for t in (ax.get_ticklabels() + ax.get_ticklines()):
+                t.set_color(self.textcolor)
         self.canvas.draw()
 
+#         fr =  axes[0].get_frame()
+#         print 'Frame'
+#         showobj(fr)
+#         print fr.set_edgecolor('#00AA22')
+#         print fr.get_edgecolor()
+#         print fr.get_axes()
+#         showobj(axes[0])
+#         #         print 'Xaxis'
+#         showobj(axes[0].xaxis)
+
+
     def refresh_trace(self,trace=None):
-        if trace is None: trace = self.ntrace
+        trace = self.__gettrace(trace)
         self.traces[trace].update(self.__mpline(trace))
 
     def set_trace_color(self,color,trace=None):
-        if trace is None:
-            trace = self.ntrace
+        trace = self.__gettrace(trace)
         self.traces[trace].set_color(color,line=self.__mpline(trace))
 
-    def set_trace_label(self,label,trace=None):
-        if trace is None: trace = self.ntrace
+    def set_trace_label(self, label, trace=None):
+        trace = self.__gettrace(trace)
         self.traces[trace].set_label(label,line=self.__mpline(trace))
 
     def set_trace_style(self,style,trace=None):
-        if trace is None: trace = self.ntrace
+        trace = self.__gettrace(trace)
         self.traces[trace].set_style(style,line=self.__mpline(trace))
 
     def set_trace_drawstyle(self, style,trace=None):
-        if trace is None: trace = self.ntrace
+        trace = self.__gettrace(trace)
         self.traces[trace].set_drawstyle(style, line=self.__mpline(trace))
 
     def set_trace_marker(self,marker,trace=None):
-        if trace is None: trace = self.ntrace
+        trace = self.__gettrace(trace)
         self.traces[trace].set_marker(marker,line=self.__mpline(trace))
 
     def set_trace_markersize(self,markersize,trace=None):
-        if trace is None: trace = self.ntrace
+        trace = self.__gettrace(trace)
         self.traces[trace].set_markersize(markersize,line=self.__mpline(trace))
 
     def set_trace_linewidth(self,linewidth,trace=None):
-        if trace is None: trace = self.ntrace
+        trace = self.__gettrace(trace)
         self.traces[trace].set_linewidth(linewidth,line=self.__mpline(trace))
 
+    def set_trace_datarange(self, datarange, trace=None):
+        trace = self.__gettrace(trace)
+        self.traces[trace].data_range = datarange
+
+    def get_trace_datarange(self, trace=None):
+        trace = self.__gettrace(trace)
+        return self.traces[trace].data_range
+
     def get_mpl_line(self,trace=None):
-        if trace is None: trace = self.ntrace
+        trace = self.__gettrace(trace)
         return self.__mpline(trace)[0]
 
+    def enable_grid(self, show=None):
+        "enable/disable grid display"
+        if show is not None:
+            self.show_grid = show
+        axes = self.canvas.figure.get_axes()
+        axes[0].grid(self.show_grid)
+        for ax in axes[1:]:
+            ax.grid(False)
+        self.canvas.draw()
+
+
+    def draw_legend(self, show=None):
+        "redraw the legend"
+        if show is not None:
+            self.show_legend = show
+
+        axes = self.canvas.figure.get_axes()
+        # clear existing legend
+        try:
+            lgn = self.mpl_legend
+            if lgn:
+                for i in lgn.get_texts():
+                    i.set_text('')
+                for i in lgn.get_lines():
+                    i.set_linewidth(0)
+                    i.set_markersize(0)
+                    i.set_marker('None')
+                lgn.draw_frame(False)
+                lgn.set_visible(False)
+        except:
+            pass
+
+        labs = []
+        lins = []
+        for ax in axes:
+            for xline in ax.get_lines():
+                if xline.get_label() != '_nolegend_':
+                    lins.append(xline)
+
+        for l in lins:
+            xl = l.get_label()
+            if not self.show_legend: xl = ''
+            labs.append(xl)
+        labs = tuple(labs)
+
+        lgn = axes[0].legend
+        if self.legend_onaxis == 'off plot':
+            lgn = self.canvas.figure.legend
+
+        if self.show_legend:
+            self.mpl_legend = lgn(lins, labs,
+                                  loc=self.legend_loc,
+                                  prop=self.labelfont)
+            self.mpl_legend.draw_frame(self.show_legend_frame)
+        self.canvas.draw()
+
