@@ -12,6 +12,7 @@ import matplotlib
 import matplotlib.cm as colormap
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
+from matplotlib.nxutils import points_inside_poly
 
 from imageconf import ImageConfig, ImageConfigFrame
 from basepanel import BasePanel
@@ -29,25 +30,35 @@ class ImagePanel(BasePanel):
     """
 
     def __init__(self, parent, messenger=None, data_callback=None,
-                 size=(4.50,4.00), dpi=96, **kws):
+                 size=(4.50,4.00), dpi=100,
+                 output_title='Image', lasso_callback=None, **kws):
         matplotlib.rc('lines', linewidth=2)
-        BasePanel.__init__(self, parent, messenger=messenger, **kws)
-        self.data_callback = data_callback
+        BasePanel.__init__(self, parent,
+                           output_title=output_title,
+                           messenger=messenger, **kws)
         self.conf = ImageConfig()
-        self.win_config = None
+        self.data_callback = data_callback
+        self.lasso_callback = lasso_callback
         self.cursor_callback = None
+        self.contour_callback = None
+        self.win_config = None
         self.figsize = size
         self.dpi     = dpi
         self.xlab    = 'X'
         self.ylab    = 'Y'
         self.xdata   = None
         self.ydata   = None
+        self.user_limits = {}
         self.BuildPanel()
 
-    def display(self, data, x=None, y=None, xlabel=None, ylabel=None, **kw):
+    def display(self, data, x=None, y=None, xlabel=None, ylabel=None,
+                style=None, nlevels=None, levels=None, contour_labels=None,
+                **kws):
         """
-        display a new image display on the current panel
+        generic display, using imshow (default) or contour
         """
+        if style is not None:
+            self.conf.style = style
         self.axes.cla()
         self.conf.rot = False
         self.data_range = [0, data.shape[1], 0, data.shape[0]]
@@ -65,24 +76,65 @@ class ImagePanel(BasePanel):
         if ylabel is not None:
             self.ylab = ylabel
         self.conf.data = data
+
+        wx.CallAfter(self.calc_indices)
         cmap = self.conf.cmap
-        img = (data -data.min()) /(1.0*data.max() - data.min())
-        self.conf.image = self.axes.imshow(img, cmap=self.conf.cmap,
-                                           interpolation=self.conf.interp)
+        if self.conf.style == 'contour':
+            if levels is None:
+                levels = self.conf.ncontour_levels
+            else:
+                self.conf.ncontour_levels = levels
+            if nlevels is None:
+                nlevels = self.conf.ncontour_levels = 9
+            nlevels = max(2, nlevels)
+            clevels  = np.linspace(data.min(), data.max(), nlevels+1)
+            self.conf.contour_levels = clevels
+            self.conf.image = self.axes.contourf(data, cmap=self.conf.cmap,
+                                                 levels=clevels)
+
+            self.conf.contour = self.axes.contour(data, cmap=self.conf.cmap,
+                                                  levels=clevels)
+            #print 'Contour levels: '
+            #print ['%.3f' % i for i in self.conf.contour_levels]
+            cmap_name = self.conf.cmap.name
+            xname = 'gray'
+            try:
+                if cmap_name == 'gray_r':
+                    xname = 'Reds_r'
+                elif cmap_name == 'gray':
+                    xname = 'Reds'
+                elif cmap_name.endswith('_r'):
+                    xname = 'gray_r'
+            except:
+                pass
+            self.conf.contour.set_cmap(getattr(colormap, xname))
+
+            if contour_labels is None:
+                contour_labels = self.conf.contour_labels
+            if contour_labels:
+                self.axes.clabel(self.conf.contour, fontsize=10, inline=1)
+            if hasattr(self.contour_callback , '__call__'):
+                self.contour_callback(levels=clevels)
+        else: # image
+            img = (data -data.min()) /(1.0*data.max() - data.min())
+            self.conf.image = self.axes.imshow(img, cmap=self.conf.cmap,
+                                               interpolation=self.conf.interp)
+
         self.axes.set_axis_off()
         self.unzoom_all()
         if hasattr(self.data_callback, '__call__'):
-            self.data_callback(data, x=x, y=y, **kw)
+            self.data_callback(data, x=x, y=y, **kws)
 
-    def set_xylims(self, lims, axes=None, autoscale=True):
+    def set_viewlimits(self, axes=None, autoscale=False):
         """ update xy limits of a plot"""
         if axes is None:
             axes = self.axes
 
-        if autoscale:
-            xmin, xmax, ymin, ymax = self.data_range
-        else:
-            xmin, xmax, ymin, ymax = lims
+        xmin, xmax, ymin, ymax = self.data_range
+        if not autoscale and len(self.zoom_lims) >1:
+            zlims = self.zoom_lims[-1]
+            if axes in zlims:
+                xmin, xmax, ymin, ymax = zlims[axes]
 
         xmin = int(max(self.data_range[0], xmin) + 0.5)
         xmax = int(min(self.data_range[1], xmax) + 0.5)
@@ -103,8 +155,7 @@ class ImagePanel(BasePanel):
         if autoscale:
             self.axes.set_xbound(self.axes.xaxis.get_major_locator().view_limits(xmin,xmax))
             self.axes.set_ybound(self.axes.yaxis.get_major_locator().view_limits(ymin,ymax))
-
-        self.conf.xylims = [[xmin, xmax], [ymin, ymax]]
+        self.conf.datalimits = [xmin, xmax, ymin, ymax]
         self.redraw()
 
     def clear(self):
@@ -123,7 +174,7 @@ class ImagePanel(BasePanel):
     def BuildPanel(self):
         """ builds basic GUI panel and popup menu"""
 
-        self.fig   = Figure(self.figsize,dpi=self.dpi)
+        self.fig   = Figure(self.figsize, dpi=self.dpi)
         self.axes  = self.fig.add_axes([0.02, 0.02, 0.96, 0.96])
 
         self.canvas = FigureCanvasWxAgg(self, -1, self.fig)
@@ -145,11 +196,67 @@ class ImagePanel(BasePanel):
 
     def BuildPopup(self):
         # build pop-up menu for right-click display
-        pass
+        self.popup_unzoom_all = wx.NewId()
+        self.popup_unzoom_one = wx.NewId()
+        self.popup_rot90     = wx.NewId()
+        self.popup_curmode   = wx.NewId()
+        self.popup_save   = wx.NewId()
+        self.popup_menu = wx.Menu()
+        self.popup_menu.Append(self.popup_unzoom_one, 'Zoom out')
+        self.popup_menu.Append(self.popup_unzoom_all, 'Zoom all the way out')
+        self.popup_menu.AppendSeparator()
+        self.popup_menu.Append(self.popup_rot90,   'Rotate 90deg (CW)')
+        # self.popup_menu.Append(self.popup_curmode, 'Toggle Cursor Mode')
+
+        #if self.show_config_popup:
+        #    self.popup_menu.Append(self.popup_config,'Configure')
+
+        self.popup_menu.Append(self.popup_save,  'Save Image')
+        self.Bind(wx.EVT_MENU, self.unzoom,       id=self.popup_unzoom_one)
+        self.Bind(wx.EVT_MENU, self.unzoom_all,   id=self.popup_unzoom_all)
+        self.Bind(wx.EVT_MENU, self.save_figure,  id=self.popup_save)
+        # self.Bind(wx.EVT_MENU, self.toggle_curmode,  id=self.popup_curmode)
+        self.Bind(wx.EVT_MENU, self.rotate90,  id=self.popup_rot90)
+
+    def rotate90(self, event=None):
+        "rotate 90 degrees, CW"
+        self.conf.rot = True
+        self.unzoom_all()
+
+    def toggle_curmode(self, event=None):
+        "toggle cursor mode"
+        if self.conf.cursor_mode == 'zoom':
+            self.conf.cursor_mode = 'lasso'
+        else:
+            self.conf.cursor_mode = 'zoom'
 
     ####
     ## GUI events, overriding BasePanel components
     ####
+    def calc_indices(self):
+        """calculates and stores the set of indices
+        ix=[0, nx-1], iy=[0, ny-1] for data of shape (nx, ny)"""
+        if self.conf.data is not None:
+            ny, nx = self.conf.data.shape
+            inds = []
+            for iy in range(ny):
+                inds.extend([(iy, ix) for ix in range(nx)])
+            self.conf.indices = np.array(inds)
+
+    def lassoHandler(self, vertices):
+        conf = self.conf
+        if conf.indices is None:
+            self.calc_indices()
+
+        ind = conf.indices
+        mask = points_inside_poly(ind, vertices)
+        sel = [(ind[i][0], ind[i][1]) for i in np.nonzero(mask)[0]]
+        self.lasso = None
+        self.canvas.draw_idle()
+        if hasattr(self.lasso_callback , '__call__'):
+            self.lasso_callback(data=conf.data, selected=sel,
+                                mask=mask)
+
     def reportMotion(self,event=None):
         pass
 
@@ -163,7 +270,8 @@ class ImagePanel(BasePanel):
             self.zoom_lims = [None]
             xmin, xmax, ymin, ymax = self.data_range
             lims = {self.axes: [xmin, xmax, ymin, ymax]}
-        self.set_xylims(lims=lims[self.axes], axes=self.axes, autoscale=False)
+        self.set_viewlimits() #   xylims(lims=lims[self.axes], axes=self.axes, autoscale=False)
+        self.canvas.draw()
 
     def unzoom_all(self, event=None):
         """ zoom out full data range """
@@ -177,6 +285,7 @@ class ImagePanel(BasePanel):
         color map
         interpolation
         """
+
         conf = self.conf
         # note: rotation re-calls display(), to reset the image
         # other transformations will just do .set_data() on image
@@ -188,29 +297,32 @@ class ImagePanel(BasePanel):
                          y=self.xdata, ylabel=self.xlab)
         # flips, log scales
         img = conf.data
-        if conf.flip_ud:   img = np.flipud(img)
-        if conf.flip_lr:   img = np.fliplr(img)
-        if conf.log_scale: img = np.log10(1.0+ 9.0 * img)
+        if self.conf.style == 'image':
+            if conf.flip_ud:   img = np.flipud(img)
+            if conf.flip_lr:   img = np.fliplr(img)
+            if conf.log_scale: img = np.log10(1 + 9.0*img)
 
         # apply intensity scale for current limited (zoomed) image
         imin = conf.int_lo
         imax = conf.int_hi
+        if conf.log_scale:
+            imin = np.log10(1 + 9.0*imin)
+            imax = np.log10(1 + 9.0*imax)
         if conf.auto_intensity:
-            ((xmin, xmax), (ymin, ymax)) = self.conf.xylims
+            (xmin, xmax, ymin, ymax) = self.conf.datalimits
             if xmin is None:  xmin = 0
             if xmax is None:  xmax = img.shape[1]
             if ymin is None:  ymin = 0
             if ymax is None:  ymax = img.shape[0]
             imin = np.min(img[ymin:ymax, xmin:xmax])
             imax = np.max(img[ymin:ymax, xmin:xmax])
-        img = (img - imin)/(imax - imin + 1.e-8)
-
         # apply clipped color scale, as from sliders
+        img = (img - imin)/(imax - imin + 1.e-8)
         mlo = conf.cmap_lo/(1.0*conf.cmap_range)
         mhi = conf.cmap_hi/(1.0*conf.cmap_range)
-        conf.image.set_data(np.clip((img - mlo)/(mhi - mlo + 1.e-8), 0, 1))
-
-        conf.image.set_interpolation(conf.interp)
+        if self.conf.style == 'image':
+            conf.image.set_data(np.clip((img - mlo)/(mhi - mlo + 1.e-8), 0, 1))
+            conf.image.set_interpolation(conf.interp)
         self.canvas.draw()
 
     def reportLeftDown(self,event=None):
